% This function prices a double digital no touch barrier option.
% Written by:   Michael Wencis, 2008
% Package:          MATLAB 7.5.0 with Statistics Toolbox
% 
% Option Definition:
% This option pays a fixed cash amount if the underlying price stays between 
% the two boundaries for the entire lifetime of the option
% 
% Parameters:
% S:      spot exchange rate
% U:      upper barrier
% L:      lower barrier
% rd:     domestic risk-free rate
% rf:     foreign risk-free rate
% sd:     volatility
% T:      time to maturity (actual/365)
% R:      rebate (assumed to be 1 when using notional amounts)
% BH:     logical.  barrier hit? (usually false for a DNT position)
%   
% function call: DoubleDigitalNoTouch(S,U,L,rd,rf,sd,T,R,BH)

function price = DoubleDigitalNoTouch(S,U,L,rd,rf,sd,T,R,BH)
% Define 
c = (2*(rd-rf)/(sd*sd))+1;
b = (rd-rf+.5*sd*sd)*T;
m = (2*(rd-rf-.5*sd*sd)/(sd*sd));

% Calculate the probability that the barrier is not touched.
ProbNoTouch = 0;
for n = -15:15
    d1 = (log((S*(U^(2*n)))/(L^(2*n + 1))) + b)/(sd*sqrt(T));
    d2 = (log((S*(U^(2*n - 1)))/(L^(2*n))) + b)/(sd*sqrt(T));
    d3 = (log((L^(2*n + 1))/(S*(U^(2*n)))) + b)/(sd*sqrt(T));
    d4 = (log((L^(2*n + 2))/(S*(U^(2*n + 1)))) + b)/(sd*sqrt(T));

    cdf12x = normcdf(d1 - sd*sqrt(T))-normcdf(d2- sd*sqrt(T));    % N(d1 - sd*sqrt(T)) - N(d2 - sd*sqrt(T))
    cdf34x = normcdf(d3- sd*sqrt(T))-normcdf(d4- sd*sqrt(T));
    
    ProbNoTouch = ProbNoTouch + (((U/L)^(n*(c-2)))*cdf12x - (((L^(n+1))/(S*(U^(n))))^(c-2))*cdf34x);
end

if(BH)
    price = 0;
else
    price = R*exp(-rd*T)*ProbNoTouch;
end