% This function prices a Double Digital One Touch barrier option
% 
% Written by:   Michael Wencis, 2008
% Package:          MATLAB 7.5.0 with Statistics Toolbox
% 
% Option Definition:
% This option, also known as an American Binary, pays a cash rebate if the
% underlying price is located outside of the two barriers during the
% option's lifetime.  The cash is paid at the time the barrier is breached.
% 
% Parameters:
% S:      spot exchange rate
% U:      upper barrier
% L:      lower barrier
% rd:     domestic risk-free rate
% rf:     foreign risk-free rate
% sd:     volatility
% T:      time to maturity (actual/365)
% R:      rebate (assumed to be 1 when using notional amounts)
% BH:     logical.  barrier hit? 
% 
% function call: DoubleDigitalOneTouch(S,U,L,rd,rf,sd,T,R,BH)

function price = DoubleDigitalOneTouch(S,U,L,rd,rf,sd,T,R,BH)
exponent = ((rd-rf)/(sd*sd))-.5;                % As defined in the ALGO documentation
k = sqrt(2*sd*sd*rd + ((rd-rf-.5*sd*sd)^(2)));  % As defined in the ALGO documentation
sum1 = 0;
sum2 = 0;
for n = -20:20
    c1 = ((S*(U^(2*n - 1)))/(L^(2*n)))^(k/(sd*sd));
    c3 = log((L^(2*n))/(S*(U^(2*n - 1))));
    c4 = ((S*(U^(2*n)))/(L^(2*n + 1)))^(k/(sd*sd));
    c6 = log((L^(2*n + 1))/(S*(U^(2*n))));
    if(n > 0)               % set up indicator variables
        indicator1 = 1;
    else
        indicator1 = 0;
    end
    if(n <= 0)
        indicator2 = 1;
    else
        indicator2 = 0;
    end
    if(n < 0)
        indicator3 = 1;
    else
        indicator3 = 0;
    end
    if(n >= 0)
        indicator4 = 1;
    else
        indicator4 = 0;
    end
    cdfc3m = normcdf((k*T - c3)/(sd*sqrt(T)));
    cdfc3 = normcdf((k*T + c3)/(sd*sqrt(T)));
    cdfc6 = normcdf((k*T + c6)/(sd*sqrt(T)));
    cdfc6m = normcdf((k*T - c6)/(sd*sqrt(T)));
    
    sum1 = sum1 + ((c1*(cdfc3m - indicator1)) - ((1/c1)*(cdfc3 - indicator2)));
    sum2 = sum2 + ((1/c4)*(cdfc6 - indicator3) - c4*(cdfc6m - indicator4));
end
UnitDOT = ((U/S)^(exponent))*sum1 + ((L/S)^(exponent))*sum2;

if(BH)
    T = 0;
    price = R*exp(-rd*T);
else
    price = exp(-rd*T)*R*UnitDOT;
end

