% This function prices a double knock out call barrier option 
%  
% Written by:   Michael Wencis, 2008
% Package:      MATLAB 7.5.0 with Statistics Toolbox
% 
% Option Definition:
% This is a standard European type option except that if the underlying
% price is loacted outside of the two barriers during the option's
% lifetime, then the option is considered to be dead or pays a cash rebate
% at the breach time.
% 
% Parameters:
% S:      spot rate
% K:      strike price
% U:      upper barrier
% L:      lower barrier
% rd:     domestic risk-free rate
% rf:     foreign risk-free rate
% sd:     volatility
% T:      time to maturity (actual/365)
% BH:     logical.  barrier hit?
%   
% function call: EuropeanDoubleKnockoutCall(S,K,U,L,rd,rf,sd,T,BH)
function price = EuropeanDoubleKnockoutCall(S,K,U,L,rd,rf,sd,T,BH)
format LONG G;
% Define
c = (2*(rd-rf)/(sd*sd))+1;
b = (rd-rf+.5*sd*sd)*T;

sum1 = 0;
sum2 = 0;
for n = -15:15
    d1 = (log((S*(U^(2*n)))/(K*(L^(2*n)))) + b)/(sd*sqrt(T));
    d2 = (log((S*(U^(2*n - 1)))/(L^(2*n))) + b)/(sd*sqrt(T));
    d3 = (log((L^(2*n + 2))/(K*S*(U^(2*n)))) + b)/(sd*sqrt(T));
    d4 = (log((L^(2*n + 2))/(S*(U^(2*n + 1)))) + b)/(sd*sqrt(T));
    
    cdf12 = normcdf(d1)-normcdf(d2);    % N(d1) - N(d2)
    cdf34 = normcdf(d3)-normcdf(d4);
    cdf12x = normcdf(d1 - sd*sqrt(T))-normcdf(d2- sd*sqrt(T));    % N(d1 - sd*sqrt(T)) - N(d2 - sd*sqrt(T))
    cdf34x = normcdf(d3- sd*sqrt(T))-normcdf(d4- sd*sqrt(T));
    
    sum1 = sum1 + (((U/L)^(n*c))*cdf12 - (((L^(n+1))/(S*(U^(n))))^(c))*cdf34);
    
    sum2 = sum2 + ((((U^(n))/(L^(n)))^(c-2))*cdf12x - (((L^(n+1))/(S*(U^(n))))^(c-2))*cdf34x);
end
if(BH)
    price = 0;
else
    price = S*exp(-rf*T)*sum1 - K*exp(-rd*T)*sum2;
end