% This function prices a double knockout put barrier option 
%  
% Written by:   Michael Wencis, 2008
% Package:      MATLAB 7.5.0 with Statistics Toolbox
% 
% Option Definition:
% This is a standard European type option except that if the underlying
% price is loacted outside of the two barriers during the option's
% lifetime, then the option is considered to be dead or pays a cash rebate
% at the breach time.
% 
% Parameters:
% S:      spot rate
% K:      strike price
% U:      upper barrier
% L:      lower barrier
% rd:     domestic risk-free rate
% rf:     foreign risk-free rate
% sd:     volatility
% T:      time to maturity (actual/365)
%   
% function call: EuropeanDoubleKnockoutPut(S,K,U,L,rd,rf,sd,T)

function price = EuropeanDoubleKnockoutPut(S,K,U,L,rd,rf,sd,T)
% Define
c = (2*(rd-rf)/(sd*sd))+1;
b = (rd-rf+.5*sd*sd)*T;

sum1 = 0;
sum2 = 0;
for n = -15:15
    y1 = (log((S*(U^(2*n)))/(L^(2*n + 1))) + b)/(sd*sqrt(T));
    y2 = (log((S*(U^(2*n)))/(K*(L^(2*n)))) + b)/(sd*sqrt(T));
    y3 = (log((L^(2*n + 1))/(S*(U^(2*n)))) + b)/(sd*sqrt(T));
    y4 = (log((L^(2*n + 2))/(K*S*(U^(2*n)))) + b)/(sd*sqrt(T));
    
    cdf12 = normcdf(y1)-normcdf(y2);    % N(y1) - N(y2)
    cdf34 = normcdf(y3)-normcdf(y4);
    cdf12x = normcdf(y1 - sd*sqrt(T))-normcdf(y2- sd*sqrt(T));    % N(d1 - sd*sqrt(T)) - N(d2 - sd*sqrt(T))
    cdf34x = normcdf(y3- sd*sqrt(T))-normcdf(y4- sd*sqrt(T));
    
    sum1 = sum1 + ((((U^(n))/(L^(n)))^(c-2))*cdf12x - (((L^(n+1))/(S*(U^(n))))^(c-2))*cdf34x);
    
    sum2 = sum2 + (((U/L)^(n*c))*cdf12 - (((L^(n+1))/(S*(U^(n))))^(c))*cdf34);
      
end
price = K*exp(-rd*T)*sum1 - S*exp(-rf*T)*sum2;